@@ -18,22 +18,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef BOX_DATA_PORTAL_H
-#define BOX_DATA_PORTAL_H
+#ifndef GLOM_MODE_DATA_BOX_DATA_PORTAL_H
+#define GLOM_MODE_DATA_BOX_DATA_PORTAL_H
 
 #include "config.h" // GLOM_ENABLE_CLIENT_ONLY
 
 #include "box_data_manyrecords.h"
 #include <glom/utility_widgets/layoutwidgetbase.h>
-#include <gtkmm/alignment.h>
-#include <gtkmm/frame.h>
-#include <gtkmm/label.h>
-
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/button.h>
-//#include <glom/mode_data/box_data_details.h>
-#include <glom/window_boxholder.h>
-#endif //GLOM_ENABLE_MAEMO
 
 
 namespace Glom
@@ -90,8 +81,17 @@ public:
    */
   void get_suitable_record_to_view_details(const Gnome::Gda::Value& primary_key_value, Glib::ustring& table_name, Gnome::Gda::Value& table_primary_key_value) const;
 
+  /** Prevent any attempts to change actual records,
+   * if the widget is just being used to enter find critera,
+   * and prevents any need for data retrieval from the database, because
+   * no data will be displayed.
+   *
+   * @param val True if find mode should be used.
+   */
+  virtual void set_find_mode(bool val = true);
+
 protected:
-  virtual type_vecLayoutFields get_fields_to_show() const; //override
+  virtual type_vecConstLayoutFields get_fields_to_show() const; //override
     
   //Implementations of pure virtual methods from Base_DB_Table_Data:
   virtual sharedptr<Field> get_field_primary_key() const;
@@ -128,25 +128,12 @@ protected:
   // must match another field in the parent table.
   sharedptr<Field> m_key_field;
   Gnome::Gda::Value m_key_value;
+
+  bool m_find_mode;
     
   type_signal_portal_record_changed m_signal_portal_record_changed;
-  
-private:
-  #ifdef GLOM_ENABLE_MAEMO
-  void on_realize();
-  void on_unrealize();
-  void on_maemo_appmenubutton_add();
-  void on_window_maemo_details_closed();
-  
-  //Each related-records portal adds its own Add Something button 
-  //to the application's AppMenu when the portal is visible.
-  Hildon::Button m_maemo_appmenubutton_add;
-  
-  Window_BoxHolder* m_window_maemo_details;
-  Box_Data_Details* m_box_maemo_details;
-  #endif //GLOM_ENABLE_MAEMO
 };
 
 } //namespace Glom
 
-#endif //BOX_DATA_PORTAL_H
+#endif // GLOM_MODE_DATA_BOX_DATA_PORTAL_H
