@@ -21,8 +21,11 @@
 #include "combo.h"
 #include <libglom/data_structure/glomconversions.h>
 #include <gtkmm/messagedialog.h>
+#include <glom/mode_data/datawidget/cellcreation.h>
 #include <glom/dialog_invalid_data.h>
+#include <glom/mode_data/datawidget/treemodel_db_withextratext.h>
 #include <libglom/data_structure/glomconversions.h>
+#include <libglom/db_utils.h>
 #include <glom/application.h>
 #include <glom/utils_ui.h>
 #include <glibmm/i18n.h>
@@ -38,100 +41,156 @@ namespace Glom
 namespace DataWidgetChildren
 {
 
-ComboGlom::ComboGlom()
-: ComboChoicesWithTreeModel(),
-  m_cell_second(0)
+ComboGlom::ComboGlom(bool has_entry)
+: Gtk::ComboBox(has_entry),
+  ComboChoicesWithTreeModel(),
+  m_ignore_changed(false)
 {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   setup_menu();
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  init();
-}
-
-void ComboGlom::init()
-{
-  #ifndef GLOM_ENABLE_MAEMO
-  set_model(m_refModel);
-  pack_start(m_Columns.m_col_first);
-  #else
-  //Maemo:
-  set_selector(m_maemo_selector);
-  m_maemo_selector.set_model(0, m_refModel);
-
-  Glib::RefPtr<Hildon::TouchSelectorColumn> column =
-    m_maemo_selector.append_text_column(m_refModel);
-  column->set_property("text-column", 0); // TODO: Add a TextSelectorColumn::set_text_column() method?
-
-  column->pack_start(m_Columns.m_col_first, false);
-  #endif //GLOM_ENABLE_MAEMO
-
   //if(m_glom_type == Field::TYPE_NUMERIC)
    // get_entry()->set_alignment(1.0); //Align numbers to the right.
+
+  //Let the combo be big enough:
+  set_popup_fixed_width(false);
 }
 
 ComboGlom::~ComboGlom()
 {
 }
 
-void ComboGlom::set_choices_related(const Document* document, const sharedptr<const Relationship>& relationship, const Glib::ustring& field, const Glib::ustring& field_second, bool show_all)
+void ComboGlom::set_choices_fixed(const FieldFormatting::type_list_values& list_values)
 {
-  //Add the extra cell if necessary:
-  if(!m_cell_second && !field_second.empty())
+  ComboChoicesWithTreeModel::set_choices_fixed(list_values);
+
+  Glib::RefPtr<Gtk::TreeModel> model = get_choices_model();
+  if(!model)
   {
-    #ifndef GLOM_ENABLE_MAEMO
-    //We don't use this convenience method, because we want more control over the renderer.
-    //and CellLayout gives no way to get the renderer back afterwards.
-    //(well, maybe set_cell_data_func(), but that's a bit awkward.)
-    //pack_start(m_Columns.m_col_second);
-
-    m_cell_second = Gtk::manage(new Gtk::CellRendererText);
-    m_cell_second->set_property("xalign", 0.0);
-
-    //Use the renderer:
-    pack_start(*m_cell_second);
-
-    //Make the renderer render the column:
-    add_attribute(m_cell_second->_property_renderable(), m_Columns.m_col_second);
-    std::cout << "debug: Added second column." << std::endl;
-    #else
-    //Maemo:
-    column->pack_start(m_Columns.m_col_second);
-    #endif //GLOM_ENABLE_MAEMO
+    std::cerr << G_STRFUNC << ": model is null." << std::endl;
+    return;
   }
 
-  ComboChoicesWithTreeModel::set_choices_related(document, relationship, field, field_second, show_all);
+  //Show the model in the view:
+  set_model(model);
+
+  if(get_has_entry())
+  {
+    set_entry_text_column( get_fixed_model_text_column() );
+  }
+  else
+  {
+    clear(); //This breaks GtkCombo with has-entry.
+  }
+
+  Glib::RefPtr<Gtk::CellAreaBox> cell_area = 
+    Glib::RefPtr<Gtk::CellAreaBox>::cast_dynamic(get_area());
+  if(!cell_area)
+  {
+    std::cerr << G_STRFUNC << ": Unexpected or null CellArea type." << std::endl;
+    return;
+  }
+  
+  guint columns_count = model->get_n_columns();
+  if(columns_count)
+    columns_count -= 1; //The last one is the just the extra text-equivalent of the first one, for GtkComboBox wth has-entry=true.
+
+  for(guint i = 0; i < columns_count; ++i)
+  {
+    //set_entry_text_column() adds its own CellRenderer,
+    //which we cannot replace without confusing (and crashing) GtkComboBox.
+    //We used the special get_fixed_model_text_column() column for that,
+    //so we don't need to add another cell renderer for the value-equivalent of that column:
+    if(i == 0 && get_has_entry())
+      continue;
+
+    Gtk::CellRendererText* cell = Gtk::manage(new Gtk::CellRendererText);
+    cell->property_xalign() = 0.0f;
+
+    //Use the renderer:
+    cell_area->pack_start(*cell, true /* expand */, true /* align */, true /* fixed */);
+
+    //Make the renderer render the column:
+    add_attribute(*cell, "text", i);
+  }
+}
+
+void ComboGlom::set_choices_related(const Document* document, const sharedptr<const LayoutItem_Field>& layout_field, const Gnome::Gda::Value& foreign_key_value)
+{
+  ComboChoicesWithTreeModel::set_choices_related(document, layout_field, foreign_key_value);
+
+  Glib::RefPtr<Gtk::TreeModel> model = get_choices_model();
+  if(!model)
+  {
+    std::cerr << G_STRFUNC << ": model is null." << std::endl;
+    return;
+  }
+
+  //Show the model in the view:
+  set_model(model);
+
+  if(get_has_entry())
+  {
+    Glib::RefPtr<DbTreeModelWithExtraText> model_db =
+      Glib::RefPtr<DbTreeModelWithExtraText>::cast_dynamic(model);
+    if(model_db)
+    {
+      const int text_col = model_db->get_text_column();
+      //const GType debug_type = model_db->get_column_type(text_col);
+      //std::cout << "DEBUG: text_col=" << text_col << ", debug_type=" << g_type_name(debug_type) << std::endl;
+      set_entry_text_column(text_col);
+    }
+    else
+    {
+      std::cerr << G_STRFUNC << ": The model is not a DbTreeModelWithExtraText." << std::endl;
+      return;
+    }
+  }
+  else
+  {
+    clear(); //This breaks GtkCombo with has-entry.
+  }
+
+  guint model_column_index = 0;
+  for(type_vec_const_layout_items::const_iterator iter = m_db_layout_items.begin(); iter != m_db_layout_items.end(); ++iter)
+  {
+    const sharedptr<const LayoutItem> layout_item = *iter;
+    if(!layout_item) //column_info.m_visible)
+    {
+      ++model_column_index;
+      continue;
+    }
+
+    //set_entry_text_column() adds its own CellRenderer,
+    //which we cannot replace without confusing (and crashing) GtkComboBox.
+    if(model_column_index == 0 && get_has_entry())
+    {
+       ++model_column_index;
+      continue;
+    }
+
+    Gtk::CellRenderer* cell = create_cell(layout_item, m_table_name, document, get_fixed_cell_height(*this));
+
+    //Add the ViewColumn:
+    if(cell)
+    {
+      //Use the renderer:
+      //We don't expand the first column, so we can align the other columns.
+      //Otherwise the other columns appear center-aligned.
+      //This bug is relevant: https://bugzilla.gnome.org/show_bug.cgi?id=629133
+      pack_start(*cell, false);
+
+      cell_connect_cell_data_func(this, cell, model_column_index);
+    }
+
+     ++model_column_index;
+  } //for
 }
 
 void ComboGlom::check_for_change()
 {
-  Glib::ustring new_text = get_text();
-  if(new_text != m_old_text)
-  {
-    //Validate the input:
-    bool success = false;
-
-    sharedptr<const LayoutItem_Field> layout_item = sharedptr<const LayoutItem_Field>::cast_dynamic(get_layout_item());
-    const Gnome::Gda::Value value = Conversions::parse_value(layout_item->get_glom_type(), new_text, layout_item->get_formatting_used().m_numeric_format, success);
-
-    if(success)
-    {
-      //Actually show the canonical text:
-      set_value(value);
-      m_signal_edited.emit(); //The text was edited, so tell the client code.
-    }
-    else
-    {
-      //Tell the user and offer to revert or try again:
-      bool revert = glom_show_dialog_invalid_data(layout_item->get_glom_type());
-      if(revert)
-      {
-        set_text(m_old_text);
-      }
-      else
-        grab_focus(); //Force the user back into the same field, so that the field can be checked again and eventually corrected or reverted.
-    }
-  }
+  m_signal_edited.emit();
 }
 
 void ComboGlom::set_value(const Gnome::Gda::Value& value)
@@ -140,7 +199,41 @@ void ComboGlom::set_value(const Gnome::Gda::Value& value)
   if(!layout_item)
     return;
 
-  set_text(Conversions::get_text_for_gda_value(layout_item->get_glom_type(), value, layout_item->get_formatting_used().m_numeric_format));
+  m_old_value = value;
+
+  Glib::RefPtr<Gtk::TreeModel> model = get_choices_model();
+  if(!model)
+  {
+    std::cerr << G_STRFUNC << ": model is null." << std::endl;
+    return;
+  }
+
+  //Avoid emiting the edited signal just because of these calls:
+  const bool old_ignore = m_ignore_changed;
+  m_ignore_changed = true;
+
+  bool found = false;
+  for(Gtk::TreeModel::iterator iter = model->children().begin(); iter != model->children().end(); ++iter)
+  {
+    const Gtk::TreeModel::Row row = *iter;
+    Gnome::Gda::Value this_value;
+    row.get_value(0, this_value);
+
+    if(this_value == value)
+    {
+      found = true;
+      set_active(iter);
+      break;
+    }
+  }
+
+  if(!found)
+  {
+    //Not found, so mark it as blank:
+    unset_active();
+  }
+
+  m_ignore_changed = old_ignore;
 
   //Show a different color if the value is numeric, if that's specified:
   if(layout_item->get_glom_type() == Field::TYPE_NUMERIC)
@@ -165,65 +258,20 @@ void ComboGlom::set_value(const Gnome::Gda::Value& value)
   }
 }
 
-void ComboGlom::set_text(const Glib::ustring& text)
-{
-  m_old_text = text;
-
-  for(Gtk::TreeModel::iterator iter = m_refModel->children().begin(); iter != m_refModel->children().end(); ++iter)
-  {
-    Glib::ustring this_text = (*iter)[m_Columns.m_col_first];
-
-    if(this_text == text)
-    {
-      #ifndef GLOM_ENABLE_MAEMO
-      set_active(iter);
-      #else
-      set_selected(iter);
-      #endif //GLOM_ENABLE_MAEMO
-      return; //success
-    }
-  }
-
-  //It's OK to pass "" to this method to unset any items:
-  if(!text.empty())
-  {
-    g_warning("ComboGlom::set_text(): no item found for: %s", text.c_str());
-  }
-
-  //Not found, so mark it as blank:
-  #ifndef GLOM_ENABLE_MAEMO
-  unset_active();
-  #else
-  unselect();
-  #endif
-}
-
 Gnome::Gda::Value ComboGlom::get_value() const
 {
-  sharedptr<const LayoutItem_Field> layout_item = sharedptr<const LayoutItem_Field>::cast_dynamic(get_layout_item());
-  bool success = false;
+   //Get the active row:
+   Gtk::TreeModel::iterator iter = get_active();
 
-  const Glib::ustring text = get_text();
-  return Conversions::parse_value(layout_item->get_glom_type(), text, layout_item->get_formatting_used().m_numeric_format, success);
-}
-
-Glib::ustring ComboGlom::get_text() const
-{
-  //Get the active row:
-  #ifndef GLOM_ENABLE_MAEMO
-  Gtk::TreeModel::iterator iter = get_active();
-  #else
-  ComboGlom* unconst = const_cast<ComboGlom*>(this);
-  Gtk::TreeModel::iterator iter = unconst->get_selected();
-  #endif //GLOM_ENABLE_MAEMO
-
-  if(iter)
-  {
-    Gtk::TreeModel::Row row = *iter;
-    return row[m_Columns.m_col_first];
+   if(iter)
+   {
+     const Gtk::TreeModel::Row row = *iter;
+     Gnome::Gda::Value value;
+     row.get_value(0, value);
+     return value;
   }
 
-  return Glib::ustring();
+  return Gnome::Gda::Value();
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -248,7 +296,7 @@ g_warning("ComboGlom::on_button_press_event()");
     if(pApp->get_userlevel() == AppState::USERLEVEL_DEVELOPER)
     {
       GdkModifierType mods;
-      gdk_window_get_pointer( gtk_widget_get_window (Gtk::Widget::gobj()), 0, 0, &mods );
+      gdk_window_get_device_position( gtk_widget_get_window (Gtk::Widget::gobj()), event->device, 0, 0, &mods );
       if(mods & GDK_BUTTON3_MASK)
       {
         //Give user choices of actions on this item:
@@ -272,22 +320,17 @@ Application* ComboGlom::get_application()
 }
 
 
-#ifndef GLOM_ENABLE_MAEMO
 void ComboGlom::on_changed()
-#else
-void ComboGlom::on_changed(int /* column */)
-#endif
 {
   //Call base class:
   Gtk::ComboBox::on_changed();
 
+  if(m_ignore_changed)
+    return;
+
   //This signal is emitted for every key press, but sometimes it's just to say that the active item has changed to "no active item",
   //if the text is not in the dropdown list:
-  #ifndef GLOM_ENABLE_MAEMO
   Gtk::TreeModel::iterator iter = get_active();
-  #else
-  Gtk::TreeModel::iterator iter = get_selected();
-  #endif //GLOM_ENABLE_MAEMO
 
   if(iter)
   {
